import React, { Component } from 'react';
import { AuctionList } from './AuctionList';
import { AuctionDetails } from './AuctionDetails';
import { NewAuction } from './NewAuction';
import { Auction } from './auction';

// Indicates which page to show. If it is the details page, the argument
// includes the specific auction to show the details of.
type Page = "list" | "new" | {kind: "details", auction: Auction}; 

type AppState = {page: Page, auctions: ReadonlyArray<Auction>};

// Whether to show debugging information in the console.
const DEBUG: boolean = true;

// Top-level component that displays the appropriate page.
export class App extends Component<{}, AppState> {

  constructor(props: {}) {
    super(props);
    this.state = {page: "list", auctions: []};
  }

  render = (): JSX.Element => {
    if (this.state.page === "list") {
      if (DEBUG) console.debug("rendering list page");
      return <AuctionList auctions={this.state.auctions}
                  onNew={this.doNewClick} onShow={this.doShowClick}/>;

    } else if (this.state.page === "new") {
      if (DEBUG) console.debug("rendering add page");
      return <NewAuction onStart={this.doStartClick} onBack={this.doBackClick}/>;

    } else {  // details
      const auction = this.state.page.auction;
      if (DEBUG) console.debug(`rendering details page for "${auction.name}"`);
      return <AuctionDetails auction={auction} onChange={this.doAuctionChange}
                  onBack={this.doBackClick}/>;
    }
  };

  doNewClick = (): void => {
    if (DEBUG) console.debug("set state to new");
    this.setState({page: "new"});
  };
  
  doShowClick = (auction: Auction): void => {
    if (DEBUG) console.debug(`set state to details for "${auction.name}"`);
    this.setState({page: {kind: "details", auction}});
  };
  
  doStartClick = (auction: Auction): void => {
    if (DEBUG) console.debug("add an auction and then show the full list");
    const auctions = this.state.auctions.concat([auction]);
    this.setState({page: "list", auctions});
  };
  
  doBackClick = (): void => {
    if (DEBUG) console.debug("set state to list");
    this.setState({page: "list"});
  };

  doAuctionChange = (oldAuction: Auction, newAuction: Auction): void => {
    // Build a new list of auctions with the old auction replaced by new one.
    const auctions = [];
    for (const auction of this.state.auctions) {
      auctions.push(auction === oldAuction ? newAuction : auction);
    }

    // Re-display the details page with the new auction.
    this.setState({auctions, page: {kind: "details", auction: newAuction}});
  };
}