import React, { Component, MouseEvent } from 'react';
import { Auction } from './auction';


type ListProps = {
  auctions: ReadonlyArray<Auction>,
  onNew: () => void,
  onShow: (auction: Auction) => void
};

type ListState = {
  now: number,  // current time when rendering
};


// Shows the list of all the auctions.
export class AuctionList extends Component<ListProps, ListState> {

  constructor(props: ListProps) {
    super(props);
    this.state = {now: Date.now()}
  }

  render = (): JSX.Element => {
    return (
      <div>
        <h2>Current Auctions</h2>
        <ul>{this.renderAuctions()}</ul>
        <button type="button" onClick={this.doRefreshClick}>Refresh</button>
        <button type="button" onClick={this.doNewClick}>New Auction</button>
      </div>);
  };

  renderAuctions = (): JSX.Element[] => {
    const auctions: JSX.Element[] = [];
    for (const auction of this.props.auctions) {
      const min = (auction.endTime - this.state.now) / 60 / 1000;
      const desc = (min < 0) ? "" :
          <span>&nbsp;&ndash; {Math.round(min)} minutes remaining</span>;
      auctions.push(
        <li key={auction.name}>
          <a href="#" onClick={(evt) => this.doAuctionClick(evt, auction)}>{auction.name}</a>
          {desc}
        </li>);
    }
    return auctions;
  };

  doRefreshClick = (_evt: MouseEvent<HTMLButtonElement>): void => {
    // Update the time at which we are showing 
    this.setState({now: Date.now()});
  };

  doNewClick = (_evt: MouseEvent<HTMLButtonElement>): void => {
    this.props.onNew();  // tell the parent to show the new auction page
  };

  doAuctionClick = (evt: MouseEvent<HTMLAnchorElement>, auction: Auction): void => {
    evt.preventDefault();
    this.props.onShow(auction);
  };
}
